!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Calculates cylindrical coordinates at arbitrary location:

!latex $(\s,\t,\z) \mapsto (R,\phi,Z)$. The lower metric elements, $g_{ij}/\sqrt g$, are also returned. 
!latex \item Need to return derivatives aswell (for vector transform in ir00aa).

!latex item This routine allows for arbitrary position, and thus must use a slow Fourier transform.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine co00aa( stz , RpZ , dR , dZ , jacobian , lgij )

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  use constants, only : zero
  use numerical, only : vsmall, small
  use fileunits, only : ounit
  use inputlist, only : Wco00aa, Nvol, Ni, Icoords
  use cputiming, only : Tco00aa
  use allglobal, only : cpus, mn, im, in, ss, Rbc, Zbs, ll, pi2nfp, myid, iss

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  LOCALS

  REAL, intent(in)  :: stz(1:3)
  REAL, intent(out) :: RpZ(1:3), dR(0:3), dZ(0:3), jacobian ! coordinates R,\phi,Z;
  REAL, intent(out) :: lgij(1:6) ! metric elements : g_{\s,\s}, g_{\s,\t}, g_{\s,\z}, g_{\t,\t}, g_{\t,\z}, g_{\z,\z}. 

  INTEGER           :: imn, Nint, ie01bgf
  REAL              :: lRmn(0:2), lZmn(0:2), lss, arg, carg, sarg

  BEGIN(co00aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  RpZ = zero ; jacobian = zero ; lgij = zero ! initialize intent(out) quantities;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( stz(1).lt.ss(ll)%s(0)-vsmall .or. stz(1).gt.ss(ll)%s(Ni(ll))+vsmall ) then
   write(ounit,1001) ll, ss(ll)%s(0) , stz(1), ss(ll)%s(Ni(ll)) , stz(1) - ss(ll)%s(0) , ss(ll)%s(Ni(ll)) - stz(1)
  endif

1001 format("co00aa : "10x"  : ll="i3" ; WARNING : s out-of-bounds ? ["f21.17" ,"f21.17" ,"f21.17" ] ; "2es13.3" ;")
  
  lss = min( max(stz(1),ss(ll)%s(0)) , ss(ll)%s(Ni(ll)) ) ! ensure that radial coordinate is within local domain;

  FATAL(co00aa,lss.lt.ss(ll)%s(0))
  FATAL(co00aa,lss.gt.ss(ll)%s(Ni(ll)))

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  dR(0:3) = zero ; dZ(0:3) = zero ! initialize summation quantities to zero;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \heading{logical control}

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  do imn = 1,mn ! loop over Fourier modes;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
   arg = im(imn)*stz(2) - in(imn)*stz(3) ; carg=cos(arg) ; sarg=sin(arg) ! Fourier argument;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   select case(Icoords)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \item \verb+if(Icoords.eq.0)+ : the coordinate interpolation is a straight linear interpolation between the interfaces, eg.
!latex \be R_j = R_{l-1,j} + \left(R_{l,j}-R_{l-1,j}\right) \frac{ ( s - s_{l-1} ) }{ ( s_{l}-s_{l-1})}.
!latex \ee
    
   case(0)

    lRmn(0) = Rbc(ll-1,0,imn) + Rbc(ll,1,imn) * ( lss - ss(ll)%s(0) )
    lRmn(1) =                   Rbc(ll,1,imn)
    lRmn(2) =                   zero
    lZmn(0) = Zbs(ll-1,0,imn) + Zbs(ll,1,imn) * ( lss - ss(ll)%s(0) )
    lZmn(1) =                   Zbs(ll,1,imn)
    lZmn(2) =                   zero
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
!latex \item \verb+if(Icoords.eq.1 .or. Icoords.eq.2)+ : 
!latex the coordinate interpolation is a piecewise cubic interpolation between the interfaces, provided by the NAG routine \verb+E01BGF+.
!latex \item Note that if \verb+Icoords.eq.2+, the interpolation derivatives are not provided by NAG but are given by finite-differences.
!latex Warning : this needs to be checked more closely.
    
   case(1:2)
    
    Nint=1 ! cubic interpolation; derivatives calculated in ih00aa;
    
    ie01bgf=1
    call E01BGF( Nvol+1 , iss(0:Nvol) , Rbc(0:Nvol,0,imn) , Rbc(0:Nvol,1,imn) , Nint , lss , lRmn(0) , lRmn(1) , ie01bgf )
    FATAL(co00aa,ie01bgf.ne.0)
    
    ie01bgf=1
    call E01BGF( Nvol+1 , iss(0:Nvol) , Zbs(0:Nvol,0,imn) , Zbs(0:Nvol,1,imn) , Nint , lss , lZmn(0) , lZmn(1) , ie01bgf )
    FATAL(co00aa,ie01bgf.ne.0)
    
   case default
    
    FATAL(co00aa : invalid Icoords,.true.)
    
   end select
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Recall that the derivatives, \verb+Rbc(:,1,:),Zbs(0:Nvol,1,1:mn)+ have been calculated in \verb+ih00aa+.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
   dR(0) = dR(0) + lRmn(0) * carg                !   R
   dR(1) = dR(1) + lRmn(1) * carg                ! d R / d s
   dR(2) = dR(2) + lRmn(0) * sarg * ( -im(imn) ) ! d R / d t
   dR(3) = dR(3) + lRmn(0) * sarg * ( +in(imn) ) ! d R / d z
   
   dZ(0) = dZ(0) + lZmn(0) * sarg                !   Z       ; only for plotting;
   dZ(1) = dZ(1) + lZmn(1) * sarg                ! d Z / d s
   dZ(2) = dZ(2) + lZmn(0) * carg * ( +im(imn) ) ! d Z / d t
   dZ(3) = dZ(3) + lZmn(0) * carg * ( -in(imn) ) ! d Z / d z
   
  enddo ! end of Fourier summation loop;

!latex \item The jacobian is \be \sqrt g = R ( R_s Z_\theta - R_\theta Z_s ). \ee

   jacobian = dR(0) * ( dR(1)*dZ(2) - dR(2)*dZ(1) )

   if( abs(jacobian).lt.vsmall ) then
    cput = MPI_WTIME() ; write(ounit,'("co00aa : "f10.2" : ll="i3" ; (s,t,z)=("es23.15" ,"es23.15" ,"es23.15" ) ; jacobian="es23.15" ;")')cput-cpus,ll,stz,jacobian
    jacobian = small
   endif

   FATAL(co00aa,jacobian.lt.vsmall)

!latex \item The `lower' metric coefficients, $g_{ij}$, are given by
!latex \be g_{\s\s} &=& R_{\s}R_{\s} + Z_{\s}Z_{\s},    \\
!latex     g_{\s\t} &=& R_{\s}R_{\t} + Z_{\s}Z_{\t},    \\
!latex     g_{\s\z} &=& R_{\s}R_{\z} + Z_{\s}Z_{\z},    \\
!latex     g_{\t\t} &=& R_{\t}R_{\t} + Z_{\t}Z_{\t},    \\
!latex     g_{\t\z} &=& R_{\t}R_{\z} + Z_{\t}Z_{\z},    \\
!latex     g_{\z\z} &=& R_{\z}R_{\z} + Z_{\z}Z_{\z}+R^2.
!latex \ee
!latex Note that the returned $g_{ij}$ are scaled by $\sqrt g$.
!latex This should be changed: better to return $g_{ij}$ without dividing by $\sqrt g$.
  
! but it is the quantity g_{ij} / sqrt g that is actually needed below . . .
  
  lgij(1) = ( dR(1)*dR(1)               + dZ(1)*dZ(1) ) / jacobian ! g_s,s / sqrt g
  lgij(2) = ( dR(1)*dR(2)               + dZ(1)*dZ(2) ) / jacobian ! g_s,t / sqrt g
  lgij(3) = ( dR(1)*dR(3)               + dZ(1)*dZ(3) ) / jacobian ! g_s,z / sqrt g
  lgij(4) = ( dR(2)*dR(2)               + dZ(2)*dZ(2) ) / jacobian ! g_t,t / sqrt g
  lgij(5) = ( dR(2)*dR(3)               + dZ(2)*dZ(3) ) / jacobian ! g_t,z / sqrt g
  lgij(6) = ( dR(3)*dR(3) + dR(0)*dR(0) + dZ(3)*dZ(3) ) / jacobian ! g_z,z / sqrt g
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  RpZ(1:3) = (/ dR(0) , -stz(3) , dZ(0) /) ! this may not really be needed; routine modified to return R,Z through dR(0),dZ(0);

  RETURN(co00aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

end subroutine co00aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
